@@ -20,17 +20,24 @@ from typing import (
     Dict,
     Type,
     Iterator,
+    Tuple,
+)
+from .c_pointer import (
+    VoidPointer,
+    TypedCPointer,
+    StructPointer,
+    _BaseCPointer,
 )
-from .c_pointer import VoidPointer, TypedCPointer, StructPointer
 import ctypes
 from . import _cstd
+from .exceptions import InvalidBindingParameter
 
 if TYPE_CHECKING:
     from .struct import Struct
 
 T = TypeVar("T")
 PointerLike = Union[TypedCPointer[Any], VoidPointer]
-StringLike = Union[str, bytes, VoidPointer]
+StringLike = Union[str, bytes, VoidPointer, TypedCPointer[bytes]]
 Format = Union[str, bytes, PointerLike]
 
 __all__ = (
@@ -142,6 +149,7 @@ __all__ = (
     "c_calloc",
     "c_realloc",
     "c_free",
+    "gmtime",
 )
 
 
@@ -153,14 +161,12 @@ def _not_null(data: Optional[T]) -> T:
 StructMap = Dict[Type[ctypes.Structure], Type["Struct"]]
 
 
-def _base(
+def _decode_response(
+    res: Any,
+    struct_map: StructMap,
     fn: "ctypes._NamedFuncPointer",
-    *args,
-    map_extra: Optional[StructMap] = None,
 ) -> Any:
-    res = fn(*args)
     res_typ = type(res)
-    struct_map: StructMap = {**STRUCT_MAP, **(map_extra or {})}
 
     if res_typ.__name__.startswith("LP_"):
         struct_type = struct_map.get(getattr(_cstd, res_typ.__name__[3:]))
@@ -171,7 +177,7 @@ def _base(
         res = (
             TypedCPointer(ctypes.addressof(res), res_typ, ctypes.sizeof(res))
             if not issubclass(type(res.contents), ctypes.Structure)
-            else StructPointer(id(struct), type(_not_null(struct)))
+            else StructPointer(id(struct), type(_not_null(struct)), struct)
         )
     # type safety gets mad if i dont use elif here
     elif fn.restype is ctypes.c_void_p:
@@ -185,11 +191,62 @@ def _base(
     return res
 
 
+def _validate_args(
+    args: Tuple[Any, ...],
+    fn: "ctypes._NamedFuncPointer",
+) -> None:
+    if not fn.argtypes:
+        return
+
+    for index, (value, typ) in enumerate(zip(args, fn.argtypes)):
+        n_type = VoidPointer.get_py(typ)
+
+        if not isinstance(value, n_type):
+            v_type = type(value)
+
+            if ((v_type is ctypes.c_char_p) and (n_type is bytes)) or (
+                issubclass(v_type, _BaseCPointer) and (typ is ctypes.c_void_p)
+            ):
+                continue
+
+            raise InvalidBindingParameter(
+                f"argument {index + 1} got invalid type: expected {n_type.__name__}, got {v_type.__name__}"  # noqa
+            )
+
+
+def _base(
+    fn: "ctypes._NamedFuncPointer",
+    *args,
+    map_extra: Optional[StructMap] = None,
+) -> Any:
+    _validate_args(args, fn)
+    res = fn(*args)
+
+    return _decode_response(
+        res,
+        {**STRUCT_MAP, **(map_extra or {})},
+        fn,
+    )
+
+
 def _make_char_pointer(data: StringLike) -> Union[bytes, ctypes.c_char_p]:
+    if type(data) not in {VoidPointer, str, bytes, TypedCPointer}:
+        raise InvalidBindingParameter(
+            f"expected a string-like object, got {repr(data)}"  # noqa
+        )
+
     if isinstance(data, bytes):
         return data
 
-    if isinstance(data, VoidPointer):
+    is_typed_ptr: bool = isinstance(data, TypedCPointer)
+
+    if isinstance(data, VoidPointer) or isinstance(data, TypedCPointer):
+        # mypy is forcing me to call this twice
+        if is_typed_ptr and (data.type is not bytes):
+            raise InvalidBindingParameter(
+                f"{data} does not point to bytes",
+            )
+
         return ctypes.c_char_p(data.address)
 
     return data.encode()
@@ -761,12 +818,18 @@ def mktime(timeptr: StructPointer[Tm]) -> int:
 
 
 def strftime(
-    string: str,
+    string: StringLike,
     maxsize: int,
-    fmt: TypedCPointer[str],
+    fmt: StringLike,
     timeptr: StructPointer[Tm],
 ) -> int:
-    return _base(dll.strftime, string, maxsize, fmt, timeptr)
+    return _base(
+        dll.strftime,
+        _make_char_pointer(string),
+        maxsize,
+        _make_char_pointer(fmt),
+        timeptr,
+    )
 
 
 def time(timer: TypedCPointer[int]) -> int:
@@ -803,3 +866,7 @@ def c_realloc(ptr: PointerLike, size: int) -> VoidPointer:
 
 def c_free(ptr: PointerLike) -> None:
     return _base(_free, ptr)
+
+
+def gmtime(timer: PointerLike) -> StructPointer[Tm]:
+    return _base(dll.gmtime, timer)
