@@ -6,6 +6,7 @@ __all__ = (
     "DereferenceError",
     "FreedMemoryError",
     "InvalidSizeError",
+    "InvalidBindingParameter",
 )  # noqa
 
 
@@ -49,3 +50,9 @@ class InvalidSizeError(Exception):
     """Raised when trying to move an object of the wrong size to an allocation."""  # noqa
 
     pass
+
+
+class InvalidBindingParameter(Exception):
+    """Raised when an invalid type is passed to the"""
+
+    pass
